@@ -934,6 +934,46 @@ bool dnsdist_ffi_set_rcode_from_async(uint16_t asyncID, uint16_t queryID, uint8_
   return dnsdist::queueQueryResumptionEvent(std::move(query));
 }
 
+static bool setAlternateName(PacketBuffer& packet, InternalQueryState& ids, std::string_view alternateName, std::string_view tag, std::string_view tagValue, std::string_view formerNameTagName)
+{
+  const DNSName originalName = ids.qname;
+  try {
+    DNSName parsed(alternateName.data(), alternateName.size(), 0, false);
+
+    // edit qname in query packet
+    if (!dnsdist::changeNameInDNSPacket(packet, originalName, parsed)) {
+      return false;
+    }
+    if (ids.d_packet) {
+      *ids.d_packet = packet;
+    }
+    // set qname to new one
+    ids.qname = std::move(parsed);
+  }
+  catch (const std::exception& e) {
+    vinfolog("Error rebasing packet on a new DNSName: %s", e.what());
+    return false;
+  }
+
+  // save existing qname in tag
+  if (!formerNameTagName.empty()) {
+    if (!ids.qTag) {
+      ids.qTag = std::make_unique<QTag>();
+    }
+    (*ids.qTag)[std::string(formerNameTagName)] = originalName.getStorage();
+  }
+
+  if (!tag.empty()) {
+    if (!ids.qTag) {
+      ids.qTag = std::make_unique<QTag>();
+    }
+    (*ids.qTag)[std::string(tag)] = tagValue;
+  }
+
+  ids.skipCache = true;
+  return true;
+}
+
 bool dnsdist_ffi_resume_from_async_with_alternate_name(uint16_t asyncID, uint16_t queryID, const char* alternateName, size_t alternateNameSize, const char* tag, size_t tagSize, const char* tagValue, size_t tagValueSize, const char* formerNameTagName, size_t formerNameTagSize)
 {
   if (!dnsdist::g_asyncHolder) {
@@ -947,59 +987,42 @@ bool dnsdist_ffi_resume_from_async_with_alternate_name(uint16_t asyncID, uint16_
   }
 
   auto& ids = query->query.d_idstate;
-  DNSName originalName = ids.qname;
-
-  try {
-    DNSName parsed(alternateName, alternateNameSize, 0, false);
-
-    PacketBuffer initialPacket;
-    if (query->d_isResponse) {
-      if (!ids.d_packet) {
-        return false;
-      }
-      initialPacket = std::move(*ids.d_packet);
-    }
-    else {
-      initialPacket = std::move(query->query.d_buffer);
-    }
-
-    // edit qname in query packet
-    if (!dnsdist::changeNameInDNSPacket(initialPacket, originalName, parsed)) {
+  PacketBuffer packet;
+  if (query->d_isResponse) {
+    if (!ids.d_packet) {
       return false;
     }
-    if (query->d_isResponse) {
-      query->d_isResponse = false;
-    }
-    query->query.d_buffer = std::move(initialPacket);
-    // set qname to new one
-    ids.qname = std::move(parsed);
+    packet = std::move(*ids.d_packet);
   }
-  catch (const std::exception& e) {
-    vinfolog("Error rebasing packet on a new DNSName: %s", e.what());
+  else {
+    packet = std::move(query->query.d_buffer);
+  }
+
+  auto wasOK = setAlternateName(packet, ids, std::string_view(alternateName, alternateNameSize), std::string_view(tag, tagSize), std::string_view(tagValue, tagValueSize), std::string_view(formerNameTagName, formerNameTagSize));
+  if (!wasOK) {
     return false;
   }
 
-  // save existing qname in tag
-  if (formerNameTagName != nullptr && formerNameTagSize > 0) {
-    if (!ids.qTag) {
-      ids.qTag = std::make_unique<QTag>();
-    }
-    (*ids.qTag)[std::string(formerNameTagName, formerNameTagSize)] = originalName.getStorage();
+  if (query->d_isResponse) {
+    query->d_isResponse = false;
   }
-
-  if (tag != nullptr && tagSize > 0) {
-    if (!ids.qTag) {
-      ids.qTag = std::make_unique<QTag>();
-    }
-    (*ids.qTag)[std::string(tag, tagSize)] = std::string(tagValue, tagValueSize);
-  }
-
-  ids.skipCache = true;
+  query->query.d_buffer = std::move(packet);
 
   // resume as query
   return dnsdist::queueQueryResumptionEvent(std::move(query));
 }
 
+bool dnsdist_ffi_dnsquestion_set_alternate_name(dnsdist_ffi_dnsquestion_t* dnsQuestion, const char* alternateName, size_t alternateNameSize, const char* tag, size_t tagSize, const char* tagValue, size_t tagValueSize, const char* formerNameTagName, size_t formerNameTagSize)
+{
+  if (dnsQuestion == nullptr || dnsQuestion->dq == nullptr || alternateName == nullptr || alternateNameSize == 0) {
+    return false;
+  }
+
+  auto& ids = dnsQuestion->dq->ids;
+  auto& packet = dnsQuestion->dq->getMutableData();
+  return setAlternateName(packet, ids, std::string_view(alternateName, alternateNameSize), std::string_view(tag, tagSize), std::string_view(tagValue, tagValueSize), std::string_view(formerNameTagName, formerNameTagSize));
+}
+
 bool dnsdist_ffi_drop_from_async(uint16_t asyncID, uint16_t queryID)
 {
   if (!dnsdist::g_asyncHolder) {
@@ -2289,7 +2312,7 @@ void dnsdist_ffi_dnsquestion_meta_begin_key([[maybe_unused]] dnsdist_ffi_dnsques
     return;
   }
 
-  dnsQuestion->pbfWriter = protozero::pbf_writer{dnsQuestion->dq->d_rawProtobufContent};
+  dnsQuestion->pbfWriter = protozero::pbf_writer{dnsQuestion->dq->ids.d_rawProtobufContent};
   dnsQuestion->pbfMetaWriter = protozero::pbf_writer{dnsQuestion->pbfWriter, static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::Field::meta)};
   dnsQuestion->pbfMetaWriter.add_string(static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaField::key), protozero::data_view(key, keyLen));
   dnsQuestion->pbfMetaValueWriter = protozero::pbf_writer {dnsQuestion->pbfMetaWriter, static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaField::value)};
@@ -2334,6 +2357,7 @@ void dnsdist_ffi_dnsquestion_meta_end_key([[maybe_unused]] dnsdist_ffi_dnsquesti
   if (dnsQuestion == nullptr) {
     return;
   }
+
   if (!dnsQuestion->pbfWriter.valid()) {
     vinfolog("Error in dnsdist_ffi_dnsquestion_meta_end_key: trying to end a key that has not been started");
     return;
@@ -2350,3 +2374,78 @@ void dnsdist_ffi_dnsquestion_meta_end_key([[maybe_unused]] dnsdist_ffi_dnsquesti
   }
 #endif /* DISABLE_PROTOBUF */
 }
+
+void dnsdist_ffi_dnsresponse_meta_begin_key([[maybe_unused]] dnsdist_ffi_dnsresponse_t* dnsResponse, [[maybe_unused]] const char* key, [[maybe_unused]] size_t keyLen)
+{
+#if !defined(DISABLE_PROTOBUF)
+  if (dnsResponse == nullptr || key == nullptr || keyLen == 0) {
+    return;
+  }
+
+  if (dnsResponse->pbfWriter.valid()) {
+    vinfolog("Error in dnsdist_ffi_dnsresponse_meta_begin_key: the previous key has not been ended");
+    return;
+  }
+
+  dnsResponse->pbfWriter = protozero::pbf_writer{dnsResponse->dr->ids.d_rawProtobufContent};
+  dnsResponse->pbfMetaWriter = protozero::pbf_writer{dnsResponse->pbfWriter, static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::Field::meta)};
+  dnsResponse->pbfMetaWriter.add_string(static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaField::key), protozero::data_view(key, keyLen));
+  dnsResponse->pbfMetaValueWriter = protozero::pbf_writer {dnsResponse->pbfMetaWriter, static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaField::value)};
+#endif /* DISABLE_PROTOBUF */
+}
+
+void dnsdist_ffi_dnsresponse_meta_add_str_value_to_key([[maybe_unused]] dnsdist_ffi_dnsresponse_t* dnsResponse, [[maybe_unused]] const char* value, [[maybe_unused]] size_t valueLen)
+{
+#if !defined(DISABLE_PROTOBUF)
+  if (dnsResponse == nullptr || value == nullptr || valueLen == 0) {
+    return;
+  }
+
+  if (!dnsResponse->pbfMetaValueWriter.valid()) {
+    vinfolog("Error in dnsdist_ffi_dnsresponse_meta_add_str_value_to_key: trying to add a value without starting a key");
+    return;
+  }
+
+  dnsResponse->pbfMetaValueWriter.add_string(static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaValueField::stringVal), protozero::data_view(value, valueLen));
+#endif /* DISABLE_PROTOBUF */
+}
+
+void dnsdist_ffi_dnsresponse_meta_add_int64_value_to_key([[maybe_unused]] dnsdist_ffi_dnsresponse_t* dnsResponse, [[maybe_unused]] int64_t value)
+{
+#if !defined(DISABLE_PROTOBUF)
+  if (dnsResponse == nullptr) {
+    return;
+  }
+
+  if (!dnsResponse->pbfMetaValueWriter.valid()) {
+    vinfolog("Error in dnsdist_ffi_dnsresponse_meta_add_int64_value_to_key: trying to add a value without starting a key");
+    return;
+  }
+
+  dnsResponse->pbfMetaValueWriter.add_uint64(static_cast<protozero::pbf_tag_type>(pdns::ProtoZero::Message::MetaValueField::intVal), value);
+#endif /* DISABLE_PROTOBUF */
+}
+
+void dnsdist_ffi_dnsresponse_meta_end_key([[maybe_unused]] dnsdist_ffi_dnsresponse_t* dnsResponse)
+{
+#if !defined(DISABLE_PROTOBUF)
+  if (dnsResponse == nullptr) {
+    return;
+  }
+
+  if (!dnsResponse->pbfWriter.valid()) {
+    vinfolog("Error in dnsdist_ffi_dnsresponse_meta_end_key: trying to end a key that has not been started");
+    return;
+  }
+
+  try {
+    /* reset the pbf writer so that the sizes are properly updated */
+    dnsResponse->pbfMetaValueWriter.commit();
+    dnsResponse->pbfMetaWriter.commit();
+    dnsResponse->pbfWriter = protozero::pbf_writer();
+  }
+  catch (const std::exception& exp) {
+    vinfolog("Error in dnsdist_ffi_dnsresponse_meta_end_key: %s", exp.what());
+  }
+#endif /* DISABLE_PROTOBUF */
+}
