@@ -279,6 +279,7 @@ IOState TCPConnectionToBackend::sendQuery(std::shared_ptr<TCPConnectionToBackend
   IOState state = conn->d_handler->tryWrite(conn->d_currentQuery.d_query.d_buffer, conn->d_currentPos, conn->d_currentQuery.d_query.d_buffer.size());
 
   if (state != IOState::Done) {
+    conn->d_lastIOBlocked = true;
     return state;
   }
 
@@ -304,12 +305,96 @@ IOState TCPConnectionToBackend::sendQuery(std::shared_ptr<TCPConnectionToBackend
   return state;
 }
 
+void TCPConnectionToBackend::handleReconnectionAttempt(std::shared_ptr<TCPConnectionToBackend>& conn, const struct timeval& now, IOStateGuard& ioGuard, IOState& iostate, bool& reconnected, bool& connectionDied)
+{
+  DEBUGLOG("connection died, number of failures is "<<conn->d_downstreamFailures<<", retries is "<<conn->d_ds->d_config.d_retries);
+
+  if (conn->d_downstreamFailures < conn->d_ds->d_config.d_retries) {
+
+    conn->d_ioState.reset();
+    ioGuard.release();
+
+    try {
+      if (conn->reconnect()) {
+        conn->d_ioState = make_unique<IOStateHandler>(*conn->d_mplexer, conn->d_handler->getDescriptor());
+
+        /* we need to resend the queries that were in flight, if any */
+        if (conn->d_state == State::sendingQueryToBackend) {
+          /* we need to edit this query so it has the correct ID */
+          auto query = std::move(conn->d_currentQuery);
+          uint16_t streamId = conn->d_highestStreamID;
+          prepareQueryForSending(query.d_query, streamId, ConnectionState::needProxy);
+          conn->d_currentQuery = std::move(query);
+        }
+
+        /* if we notify the sender it might terminate us so we need to move these first */
+        auto pendingResponses = std::move(conn->d_pendingResponses);
+        conn->d_pendingResponses.clear();
+        for (auto& pending : pendingResponses) {
+          --conn->d_ds->outstanding;
+
+          if (pending.second.d_query.isXFR() && pending.second.d_query.d_xfrStarted) {
+            /* this one can't be restarted, sorry */
+            DEBUGLOG("A XFR for which a response has already been sent cannot be restarted");
+            try {
+              TCPResponse response(std::move(pending.second.d_query));
+              pending.second.d_sender->notifyIOError(now, std::move(response));
+            }
+            catch (const std::exception& exp) {
+              vinfolog("Got an exception while notifying: %s", exp.what());
+            }
+            catch (...) {
+              vinfolog("Got exception while notifying");
+            }
+          }
+          else {
+            conn->d_pendingQueries.push_back(std::move(pending.second));
+          }
+        }
+        conn->d_currentPos = 0;
+
+        if (conn->d_state == State::sendingQueryToBackend) {
+          iostate = IOState::NeedWrite;
+          // resume sending query
+        }
+        else {
+          if (conn->d_pendingQueries.empty()) {
+            throw std::runtime_error("TCP connection to a backend in state " + std::to_string((int)conn->d_state) + " with no pending queries");
+          }
+
+          iostate = queueNextQuery(conn);
+        }
+
+        reconnected = true;
+        connectionDied = false;
+      }
+    }
+    catch (const std::exception& exp) {
+      // reconnect might throw on failure, let's ignore that, we just need to know
+      // it failed
+    }
+  }
+
+  if (!reconnected) {
+    /* reconnect failed, we give up */
+    DEBUGLOG("reconnect failed, we give up");
+    ++conn->d_ds->tcpGaveUp;
+    conn->notifyAllQueriesFailed(now, FailureReason::gaveUp);
+  }
+}
+
+
 void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& conn, const struct timeval& now)
 {
   if (conn->d_handler == nullptr) {
     throw std::runtime_error("No downstream socket in " + std::string(__PRETTY_FUNCTION__) + "!");
   }
 
+  if (conn->d_handlingIO) {
+    return;
+  }
+  dnsdist::tcp::HandlingIOGuard reentryGuard(conn->d_handlingIO);
+
   bool connectionDied = false;
   IOState iostate = IOState::Done;
   IOStateGuard ioGuard(conn->d_ioState);
@@ -317,6 +402,7 @@ void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& c
 
   do {
     reconnected = false;
+    conn->d_lastIOBlocked = false;
 
     try {
       if (conn->d_state == State::sendingQueryToBackend) {
@@ -352,8 +438,11 @@ void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& c
           conn->d_currentPos = 0;
           conn->d_lastDataReceivedTime = now;
         }
-        else if (conn->d_state == State::waitingForResponseFromBackend && conn->d_currentPos > 0) {
-          conn->d_state = State::readingResponseSizeFromBackend;
+        else {
+          conn->d_lastIOBlocked = true;
+          if (conn->d_state == State::waitingForResponseFromBackend && conn->d_currentPos > 0) {
+            conn->d_state = State::readingResponseSizeFromBackend;
+          }
         }
       }
 
@@ -373,6 +462,9 @@ void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& c
             return;
           }
         }
+        else {
+          conn->d_lastIOBlocked = true;
+        }
       }
 
       if (conn->d_state != State::idle &&
@@ -408,81 +500,7 @@ void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& c
     }
 
     if (connectionDied) {
-
-      DEBUGLOG("connection died, number of failures is "<<conn->d_downstreamFailures<<", retries is "<<conn->d_ds->d_config.d_retries);
-
-      if (conn->d_downstreamFailures < conn->d_ds->d_config.d_retries) {
-
-        conn->d_ioState.reset();
-        ioGuard.release();
-
-        try {
-          if (conn->reconnect()) {
-            conn->d_ioState = make_unique<IOStateHandler>(*conn->d_mplexer, conn->d_handler->getDescriptor());
-
-            /* we need to resend the queries that were in flight, if any */
-            if (conn->d_state == State::sendingQueryToBackend) {
-              /* we need to edit this query so it has the correct ID */
-              auto query = std::move(conn->d_currentQuery);
-              uint16_t id = conn->d_highestStreamID;
-              prepareQueryForSending(query.d_query, id, ConnectionState::needProxy);
-              conn->d_currentQuery = std::move(query);
-            }
-
-            /* if we notify the sender it might terminate us so we need to move these first */
-            auto pendingResponses = std::move(conn->d_pendingResponses);
-            conn->d_pendingResponses.clear();
-            for (auto& pending : pendingResponses) {
-              --conn->d_ds->outstanding;
-
-              if (pending.second.d_query.isXFR() && pending.second.d_query.d_xfrStarted) {
-                /* this one can't be restarted, sorry */
-                DEBUGLOG("A XFR for which a response has already been sent cannot be restarted");
-                try {
-                  TCPResponse response(std::move(pending.second.d_query));
-                  pending.second.d_sender->notifyIOError(now, std::move(response));
-                }
-                catch (const std::exception& e) {
-                  vinfolog("Got an exception while notifying: %s", e.what());
-                }
-                catch (...) {
-                  vinfolog("Got exception while notifying");
-                }
-              }
-              else {
-                conn->d_pendingQueries.push_back(std::move(pending.second));
-              }
-            }
-            conn->d_currentPos = 0;
-
-            if (conn->d_state == State::sendingQueryToBackend) {
-              iostate = IOState::NeedWrite;
-              // resume sending query
-            }
-            else {
-              if (conn->d_pendingQueries.empty()) {
-                throw std::runtime_error("TCP connection to a backend in state " + std::to_string((int)conn->d_state) + " with no pending queries");
-              }
-
-              iostate = queueNextQuery(conn);
-            }
-
-            reconnected = true;
-            connectionDied = false;
-          }
-        }
-        catch (const std::exception& e) {
-          // reconnect might throw on failure, let's ignore that, we just need to know
-          // it failed
-        }
-      }
-
-      if (!reconnected) {
-        /* reconnect failed, we give up */
-        DEBUGLOG("reconnect failed, we give up");
-        ++conn->d_ds->tcpGaveUp;
-        conn->notifyAllQueriesFailed(now, FailureReason::gaveUp);
-      }
+      handleReconnectionAttempt(conn, now, ioGuard, iostate, reconnected, connectionDied);
     }
 
     if (conn->d_ioState) {
@@ -506,7 +524,7 @@ void TCPConnectionToBackend::handleIO(std::shared_ptr<TCPConnectionToBackend>& c
       }
     }
   }
-  while (reconnected);
+  while (reconnected || (iostate != IOState::Done && !conn->d_connectionDied && !conn->d_lastIOBlocked));
 
   ioGuard.release();
 }
@@ -760,16 +778,19 @@ IOState TCPConnectionToBackend::handleResponse(std::shared_ptr<TCPConnectionToBa
     DEBUGLOG("still have some queries to send");
     return queueNextQuery(shared);
   }
-  else if (!d_pendingResponses.empty()) {
+  if (d_state == State::sendingQueryToBackend) {
+    DEBUGLOG("still have a query to send");
+    return IOState::NeedWrite;
+  }
+  if (!d_pendingResponses.empty()) {
     DEBUGLOG("still have some responses to read");
     return IOState::NeedRead;
   }
-  else {
-    DEBUGLOG("nothing to do, waiting for a new query");
-    d_state = State::idle;
-    t_downstreamTCPConnectionsManager.moveToIdle(conn);
-    return IOState::Done;
-  }
+
+  DEBUGLOG("nothing to do, waiting for a new query");
+  d_state = State::idle;
+  t_downstreamTCPConnectionsManager.moveToIdle(conn);
+  return IOState::Done;
 }
 
 uint16_t TCPConnectionToBackend::getQueryIdFromResponse() const
