@@ -1197,26 +1197,6 @@ bool IncomingTCPConnectionState::readIncomingQuery(const timeval& now, IOState&
   return false;
 }
 
-class HandlingIOGuard
-{
-public:
-  HandlingIOGuard(bool& handlingIO) :
-    d_handlingIO(handlingIO)
-  {
-  }
-  HandlingIOGuard(const HandlingIOGuard&) = delete;
-  HandlingIOGuard(HandlingIOGuard&&) = delete;
-  HandlingIOGuard& operator=(const HandlingIOGuard& rhs) = delete;
-  HandlingIOGuard& operator=(HandlingIOGuard&&) = delete;
-  ~HandlingIOGuard()
-  {
-    d_handlingIO = false;
-  }
-
-private:
-  bool& d_handlingIO;
-};
-
 void IncomingTCPConnectionState::handleIO()
 {
   // let's make sure we are not already in handleIO() below in the stack:
@@ -1227,8 +1207,7 @@ void IncomingTCPConnectionState::handleIO()
   if (d_handlingIO) {
     return;
   }
-  d_handlingIO = true;
-  HandlingIOGuard reentryGuard(d_handlingIO);
+  dnsdist::tcp::HandlingIOGuard reentryGuard(d_handlingIO);
 
   // why do we loop? Because the TLS layer does buffering, and thus can have data ready to read
   // even though the underlying socket is not ready, so we need to actually ask for the data first
@@ -1713,10 +1692,16 @@ static void tcpClientThread(pdns::channel::Receiver<ConnectionInfo>&& queryRecei
     timeval now{};
     gettimeofday(&now, nullptr);
     time_t lastTimeoutScan = now.tv_sec;
+    time_t lastConfigRefresh = now.tv_sec;
 
     for (;;) {
       data.mplexer->run(&now);
 
+      if (now.tv_sec > lastConfigRefresh) {
+        lastConfigRefresh = now.tv_sec;
+        dnsdist::configuration::refreshLocalRuntimeConfiguration();
+      }
+
       try {
         t_downstreamTCPConnectionsManager.cleanupClosedConnections(now);
         dnsdist::IncomingConcurrentTCPConnectionsManager::cleanup(time(nullptr));
@@ -1862,6 +1847,7 @@ void tcpAcceptorThread(const std::vector<ClientState*>& states)
 
   if (params.size() == 1) {
     while (true) {
+      dnsdist::configuration::refreshLocalRuntimeConfiguration();
       acceptNewConnection(params.at(0), nullptr);
     }
   }
@@ -1878,8 +1864,14 @@ void tcpAcceptorThread(const std::vector<ClientState*>& states)
     }
 
     timeval now{};
+    time_t lastConfigRefresh = now.tv_sec;
     while (true) {
       mplexer->run(&now, -1);
+
+      if (now.tv_sec > lastConfigRefresh) {
+        lastConfigRefresh = now.tv_sec;
+        dnsdist::configuration::refreshLocalRuntimeConfiguration();
+      }
     }
   }
 }
