@@ -1037,7 +1037,6 @@ BOOST_AUTO_TEST_CASE(test_SVC_Generation)
 #if !defined(DISABLE_PROTOBUF)
 BOOST_AUTO_TEST_CASE(test_meta_values)
 {
-
   InternalQueryState ids;
   ids.origRemote = ComboAddress("192.0.2.1:4242");
   ids.origDest = ComboAddress("192.0.2.255:53");
@@ -1053,6 +1052,8 @@ BOOST_AUTO_TEST_CASE(test_meta_values)
 
   DNSQuestion dnsQuestion(ids, query);
   dnsdist_ffi_dnsquestion_t lightDQ(&dnsQuestion);
+  DNSResponse dnsResponse(ids, query, nullptr);
+  dnsdist_ffi_dnsresponse_t lightDR(&dnsResponse);
 
   {
     /* check invalid parameters */
@@ -1064,18 +1065,28 @@ BOOST_AUTO_TEST_CASE(test_meta_values)
     dnsdist_ffi_dnsquestion_meta_add_str_value_to_key(&lightDQ, "some-str-value", 0);
     dnsdist_ffi_dnsquestion_meta_add_int64_value_to_key(nullptr, 0);
     dnsdist_ffi_dnsquestion_meta_end_key(nullptr);
+
+    dnsdist_ffi_dnsresponse_meta_begin_key(nullptr, nullptr, 0);
+    dnsdist_ffi_dnsresponse_meta_begin_key(&lightDR, nullptr, 0);
+    dnsdist_ffi_dnsresponse_meta_begin_key(&lightDR, "some-key", 0);
+    dnsdist_ffi_dnsresponse_meta_add_str_value_to_key(nullptr, nullptr, 0);
+    dnsdist_ffi_dnsresponse_meta_add_str_value_to_key(&lightDR, nullptr, 0);
+    dnsdist_ffi_dnsresponse_meta_add_str_value_to_key(&lightDR, "some-str-value", 0);
+    dnsdist_ffi_dnsresponse_meta_add_int64_value_to_key(nullptr, 0);
+    dnsdist_ffi_dnsresponse_meta_end_key(nullptr);
   }
 
   {
     /* trying to end a key that has not been started */
     dnsdist_ffi_dnsquestion_meta_end_key(&lightDQ);
+    dnsdist_ffi_dnsresponse_meta_end_key(&lightDR);
   }
 
   {
     const std::string key{"some-key"};
     const std::string value1{"first value"};
     const std::string value2{"second value"};
-    BOOST_CHECK_EQUAL(dnsQuestion.d_rawProtobufContent.size(), 0U);
+    BOOST_CHECK_EQUAL(dnsQuestion.ids.d_rawProtobufContent.size(), 0U);
     dnsdist_ffi_dnsquestion_meta_begin_key(&lightDQ, key.data(), key.size());
     /* we should not be able to begin a new key without ending it first */
     dnsdist_ffi_dnsquestion_meta_begin_key(&lightDQ, key.data(), key.size());
@@ -1084,10 +1095,80 @@ BOOST_AUTO_TEST_CASE(test_meta_values)
     dnsdist_ffi_dnsquestion_meta_add_str_value_to_key(&lightDQ, value2.data(), value2.size());
     dnsdist_ffi_dnsquestion_meta_add_int64_value_to_key(&lightDQ, -42);
     dnsdist_ffi_dnsquestion_meta_end_key(&lightDQ);
-    BOOST_CHECK_EQUAL(dnsQuestion.d_rawProtobufContent.size(), 55U);
-    BOOST_CHECK_EQUAL(Base64Encode(dnsQuestion.d_rawProtobufContent), "sgE0Cghzb21lLWtleRIoCgtmaXJzdCB2YWx1ZRAqCgxzZWNvbmQgdmFsdWUQ1v//////////AQ==");
+    BOOST_CHECK_EQUAL(dnsQuestion.ids.d_rawProtobufContent.size(), 55U);
+    BOOST_CHECK_EQUAL(Base64Encode(dnsQuestion.ids.d_rawProtobufContent), "sgE0Cghzb21lLWtleRIoCgtmaXJzdCB2YWx1ZRAqCgxzZWNvbmQgdmFsdWUQ1v//////////AQ==");
+    BOOST_CHECK_EQUAL(dnsResponse.ids.d_rawProtobufContent.size(), 55U);
+    BOOST_CHECK_EQUAL(Base64Encode(dnsResponse.ids.d_rawProtobufContent), "sgE0Cghzb21lLWtleRIoCgtmaXJzdCB2YWx1ZRAqCgxzZWNvbmQgdmFsdWUQ1v//////////AQ==");
+  }
+
+  {
+    const std::string key{"some-key"};
+    const std::string value1{"first value"};
+    const std::string value2{"second value"};
+    /* be careful: IDS is shared with the DNS question */
+    dnsResponse.ids.d_rawProtobufContent.clear();
+    dnsdist_ffi_dnsresponse_meta_begin_key(&lightDR, key.data(), key.size());
+    /* we should not be able to begin a new key without ending it first */
+    dnsdist_ffi_dnsresponse_meta_begin_key(&lightDR, key.data(), key.size());
+    dnsdist_ffi_dnsresponse_meta_add_str_value_to_key(&lightDR, value1.data(), value1.size());
+    dnsdist_ffi_dnsresponse_meta_add_int64_value_to_key(&lightDR, 42);
+    dnsdist_ffi_dnsresponse_meta_add_str_value_to_key(&lightDR, value2.data(), value2.size());
+    dnsdist_ffi_dnsresponse_meta_add_int64_value_to_key(&lightDR, -42);
+    dnsdist_ffi_dnsresponse_meta_end_key(&lightDR);
+    BOOST_CHECK_EQUAL(dnsResponse.ids.d_rawProtobufContent.size(), 55U);
+    BOOST_CHECK_EQUAL(Base64Encode(dnsResponse.ids.d_rawProtobufContent), "sgE0Cghzb21lLWtleRIoCgtmaXJzdCB2YWx1ZRAqCgxzZWNvbmQgdmFsdWUQ1v//////////AQ==");
   }
 }
 #endif /* DISABLE_PROTOBUF */
 
+BOOST_AUTO_TEST_CASE(test_set_altername_name)
+{
+  const DNSName initialQName("www.powerdns.com.");
+  InternalQueryState ids;
+  ids.origRemote = ComboAddress("192.0.2.1:4242");
+  ids.origDest = ComboAddress("192.0.2.255:53");
+  ids.qtype = QType::A;
+  ids.qclass = QClass::IN;
+  ids.protocol = dnsdist::Protocol::DoUDP;
+  ids.qname = initialQName;
+  ids.queryRealTime.start();
+  PacketBuffer query;
+  GenericDNSPacketWriter<PacketBuffer> pwQ(query, ids.qname, QType::A, QClass::IN, 0);
+  pwQ.getHeader()->rd = 1;
+  pwQ.getHeader()->id = htons(42);
+
+  DNSQuestion dnsQuestion(ids, query);
+  dnsdist_ffi_dnsquestion_t lightDQ(&dnsQuestion);
+
+  {
+    /* check invalid parameters */
+    dnsdist_ffi_dnsquestion_set_alternate_name(nullptr, nullptr, 0, nullptr, 0, nullptr, 0, nullptr, 0);
+    dnsdist_ffi_dnsquestion_set_alternate_name(&lightDQ, nullptr, 0, nullptr, 0, nullptr, 0, nullptr, 0);
+    dnsdist_ffi_dnsquestion_set_alternate_name(&lightDQ, "alternate", 0, nullptr, 0, nullptr, 0, nullptr, 0);
+  }
+
+  const std::string tag("alternate-name-tag");
+  const std::string tagValue("alternate-name-tag-value");
+  const std::string formerTagName("alternate-name-former-value");
+  const DNSName target("new.target.net.");
+  BOOST_REQUIRE(dnsdist_ffi_dnsquestion_set_alternate_name(&lightDQ, target.getStorage().data(), target.getStorage().size(), tag.data(), tag.size(), tagValue.data(), tagValue.size(), formerTagName.data(), formerTagName.size()));
+
+  BOOST_CHECK_EQUAL(ids.qname.toString(), target.toString());
+  BOOST_CHECK_EQUAL(ids.skipCache, true);
+  BOOST_REQUIRE(ids.qTag != nullptr);
+  BOOST_CHECK_EQUAL(ids.qTag->at(tag), tagValue);
+  BOOST_CHECK_EQUAL(ids.qTag->at(formerTagName), std::string(initialQName.getStorage()));
+
+  // NOLINTNEXTLINE(cppcoreguidelines-pro-type-reinterpret-cast)
+  MOADNSParser mdp(false, reinterpret_cast<const char*>(dnsQuestion.getData().data()), dnsQuestion.getData().size());
+  BOOST_CHECK_EQUAL(mdp.d_qname, target);
+  BOOST_CHECK_EQUAL(mdp.d_header.qdcount, 1U);
+
+  /* also check that we can rename without providing any tags */
+  BOOST_CHECK(dnsdist_ffi_dnsquestion_set_alternate_name(&lightDQ, target.getStorage().data(), target.getStorage().size(), nullptr, 0, nullptr, 0, nullptr, 0));
+
+  /* and that we can pass a tag name without a value */
+  BOOST_CHECK(dnsdist_ffi_dnsquestion_set_alternate_name(&lightDQ, target.getStorage().data(), target.getStorage().size(), tag.data(), tag.size(), nullptr, 0, nullptr, 0));
+}
+
 BOOST_AUTO_TEST_SUITE_END();
